@@ -30,7 +30,7 @@ LOG = logging.getLogger(__name__)
 OPTS = [
     cfg.StrOpt('libvirt_type',
                default='kvm',
-               choices=['kvm', 'lxc', 'qemu', 'uml', 'xen'],
+               choices=['kvm', 'lxc', 'qemu', 'uml'],
                help='Libvirt domain type.'),
     cfg.StrOpt('libvirt_uri',
                default='',
@@ -38,7 +38,7 @@ OPTS = [
                     '(which is dependent on libvirt_type).'),
 ]
 
-LIBVIRT_PER_TYPE_URIS = dict(uml='uml:///system', xen='xen:///', lxc='lxc:///')
+LIBVIRT_PER_TYPE_URIS = dict(uml='uml:///system', lxc='lxc:///')
 
 
 # We don't use the libvirt constants in case of libvirt is not available
@@ -114,7 +114,8 @@ def is_disconnection_exception(e):
 
 retry_on_disconnect = tenacity.retry(
     retry=tenacity.retry_if_exception(is_disconnection_exception),
-    stop=tenacity.stop_after_attempt(2))
+    stop=tenacity.stop_after_attempt(3),
+    wait=tenacity.wait_exponential(multiplier=3, min=1, max=60))
 
 
 def raise_nodata_if_unsupported(method):
