@@ -16,15 +16,13 @@ import abc
 from unittest import mock
 
 from oslotest import base
-import six
-from six.moves.urllib import parse as url_parse
+from urllib import parse as urlparse
 
 from ceilometer.network.statistics.opendaylight import driver
 from ceilometer import service
 
 
-@six.add_metaclass(abc.ABCMeta)
-class _Base(base.BaseTestCase):
+class _Base(base.BaseTestCase, metaclass=abc.ABCMeta):
 
     @abc.abstractproperty
     def flow_data(self):
@@ -58,20 +56,20 @@ class _Base(base.BaseTestCase):
     def inactive_hosts_data(self):
         pass
 
-    fake_odl_url = url_parse.ParseResult('opendaylight',
-                                         'localhost:8080',
-                                         'controller/nb/v2',
-                                         None,
-                                         None,
-                                         None)
+    fake_odl_url = urlparse.ParseResult('opendaylight',
+                                        'localhost:8080',
+                                        'controller/nb/v2',
+                                        None,
+                                        None,
+                                        None)
 
-    fake_params = url_parse.parse_qs('user=admin&password=admin&scheme=http&'
-                                     'container_name=default&auth=basic')
+    fake_params = urlparse.parse_qs('user=admin&password=admin&scheme=http&'
+                                    'container_name=default&auth=basic')
 
     fake_params_multi_container = (
-        url_parse.parse_qs('user=admin&password=admin&scheme=http&'
-                           'container_name=first&container_name=second&'
-                           'auth=basic'))
+        urlparse.parse_qs('user=admin&password=admin&scheme=http&'
+                          'container_name=first&container_name=second&'
+                          'auth=basic'))
 
     def setUp(self):
         super(_Base, self).setUp()
